//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.6

import wesual.Ui     1.0
import wesual.Create 1.0

import wesual.data.Core 1.0

Item {
    id : deleter

    property list<Document> documents
    property var            uncheckedTypes : []

    property string displayProperty : "label"

    function deleteDocuments() {
        var deleteDocs = [];

        var addRecursive = function(list, doc) {
            list.push(doc);

            if ("childFolders" in doc) {
                for (var i = 0; i < doc.childFolders.length; ++i) {
                    addRecursive(list, doc.childFolders[i]);
                }
            }

            if ("assets" in doc) {
                for (i = 0; i < doc.assets.length; ++i) {
                    addRecursive(list, doc.assets[i]);
                }
            }
        };

        for (var i = 0; i < documents.length; ++i) {
            addRecursive(deleteDocs, documents[i]);
        }

        refCounterComponent.createObject(deleter, {
             "documents"      : deleteDocs,
             "uncheckedTypes" : deleter.uncheckedTypes
         });
    }

    Component {
        id : refCounterComponent

        ReferenceCounter {
            id : refCounter

            property var uncheckedTypes : []
            property var referenced
            property var unreferenced

            function isUnchecked(doc) {
                return uncheckedTypes.indexOf(doc.type) !== -1;
            }

            function aggregateReferenced() {
                var ref   = [];
                var unref = [];
                for (var i = 0; i < documents.length; ++i) {
                    var doc = documents[i];
                    if (!isReferenced(doc) || isUnchecked(doc)) {
                        unref.push(doc);
                    } else {
                        ref.push(doc);
                    }
                }

                if (ref.length === 0) {
                    // No referenced documents - just delete them
                    Creator.context.beginRecordChanges();
                    for (i = 0; i < unref.length; ++i) {
                        unref[i].deleted = true;
                    }
                    Creator.context.endRecordChanges();

                    destroy();
                } else {
                    referenced   = ref;
                    unreferenced = unref;

                    messageBox.notify(refCounter);
                }
            }

            function deleteAll() {
                Creator.context.beginRecordChanges();
                for (i = 0; i < unref.length; ++i) {
                    documents[i].deleted = true;
                }
                Creator.context.endRecordChanges();
            }

            function deleteUnreferenced() {
                Creator.context.beginRecordChanges();
                for (i = 0; i < unreferenced.length; ++i) {
                    unreferenced[i].deleted = true;
                }
                Creator.context.endRecordChanges();
            }

            onStatusChanged : {
                if (status === ReferenceCounter.Ready) {
                    aggregateReferenced();
                }
            }
        }
    }

    UiMessageBox {
        id : messageBox

        property ReferenceCounter referenceCounter

        function notify(counter) {
            referenceCounter = counter;

            show();
        }

        onAccepted : referenceCounter.destroy()
        onRejected : referenceCounter.destroy()

        buttons : [
            UiDialogButton {
                text    : "Delete all"
                onClicked : {
                    messageBox.referenceCounter.deleteAll();
                }
            },
            UiDialogButton {
                text    : "Delete unreferenced"
                onClicked : {
                    messageBox.referenceCounter.deleteUnreferenced();
                }
            },
            UiDialogButton {
                role : UiDialogButton.CancelRole
            }
        ]
    }
}
