//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.5
import QtQuick.Layouts 1.2

import wesual.Ui     1.0
import wesual.Create 1.0
import wesual.Stage  1.0
import wesual.Editor 1.0

import wesual.data.Core   1.0

import wesual.module.MediaManager 1.0

RowLayout {
    id : mediaPicker

    property Asset mediaAsset

    signal mediaPicked(var media)
    signal removeMedia()

    spacing : 10

    QtObject {
        id : media

        function show() {
            var dialog = ModuleManager.globalInstance(
                        "wesual.dialog.MediaLocation");

            function cb(result) {
                mediaPicker.mediaPicked(result[0]);
            }

            dialog.open({
                callback : cb
            });
        }
    }

    Item {
        width  : 120
        height : 120

        Rectangle {
            anchors.fill : parent
            visible      : !mediaAsset
            border {
                width : 1
                color : UiColors.getColor(UiColors.SeparatorLineGrey17)
            }
            UiToolButton {
                anchors.fill : parent
                iconName     : "add"
                onClicked    : media.show()
            }
        }

        AssetThumbnail {
            id : thumbnail

            asset   : mediaAsset
            visible : mediaAsset

            UiContextMenu {
                UiMenuItem {
                    text        : qsTrId("action.Share")
                    iconName    : "fb-settings"
                    enabled     : mediaAsset
                    onTriggered : shareButton.shareActions[0].trigger()
                }
                UiMenuItem {
                    text        : qsTrId("action.EditImage")
                    iconName    : "edit-image"
                    enabled     : mediaAsset
                    onTriggered : effectEditor.show()
                }
                UiMenuItem {
                    iconName    : "exchange-image"
                    text        : qsTrId("action.ExchangeImage")
                    onTriggered : media.show()
                }
                UiMenuItem {
                    text        : qsTrId("action.Export")
                    iconName    : "export"
                    enabled     : mediaAsset
                    onTriggered : exporter.exportImages()
                }
                UiMenuItem {
                    text        : qsTrId("action.RemoveImage")
                    iconName    : "remove"
                    enabled     : mediaAsset
                    onTriggered : removeMedia()
                }
            }
            UiTooltip {
                id : tooltip

                text : {
                    if (!mediaAsset)
                        return "";

                    var text = mediaAsset.fileName;
                    var size = mediaAsset.size;
                    text += " (" + size.width + "x" + size.height + ")";

                    return text;
                }
            }
        }
    }

    ColumnLayout {
        Layout.alignment : Qt.AlignTop

        SocialShareButton {
            id : shareButton

            documents : mediaAsset ? mediaAsset : null
            enabled   : mediaAsset
            visible   : false
        }
        UiToolButton {
            iconName  : "edit-image"
            text      : qsTrId("action.EditImage")
            enabled   : mediaAsset !== null
            onClicked : effectEditor.show()
        }
        UiToolButton {
            iconName  : "exchange-image"
            text      : qsTrId("action.ExchangeImage")
            onClicked : media.show()
        }
    }

    Item {
        Layout.fillWidth : true

        ImageExporter {
            id : exporter

            assets : [ mediaAsset ]
        }
    }

    ImageEffectEditor {
        id : effectEditor

        image  : mediaAsset
    }
}
