//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.3
import QtGraphicalEffects 1.0

import wesual.Showcase 1.0
import wesual.Controls 1.0
import wesual.data.Core 1.0

InlineEditor {
    id : imageBox

    property ImageConfiguration configuration
    property Item               mask

    Connections {
        target : imageBox.configuration
        onAssetChanged : {
            if (!imageBox.configuration.asset) {
                if (effect.first) {
                    var curr = effect.first;
                    effect.first = null;
                    curr.destroy();
                }
                p_.editConfiguration = null;

                return;
            }

            loader.target = imageContext.createObject(imageBox, {
                "configuration" : imageBox.configuration
            });
            p_.editConfiguration = imageBox.configuration;
        }
    }

    QtObject {
        id : p_

        property ImageConfiguration editConfiguration
    }

    Component {
        id : imageContext

        TextureSource {
            id : source

            property alias configuration : params.configuration

            readonly property alias subrect : params.subrect

            ImageParameters {
                id : params
                boxWidth : imageBox.width
                boxHeight : imageBox.height
            }
            url : params.image
            mipmap : true
            requiredSize {
                width : params.imageWidth
                height : params.imageHeight
            }
        }
    }

    // enabled true for hover events
//    enabled : p_.editConfiguration && p_.editConfiguration.document

    Connections {
        id : loader

        ignoreUnknownSignals : true
        onTextureReady : {
            var current = effect.first;
            effect.first = target;
            effect.firstSubrect = Qt.binding(function() {
                return effect.first.subrect;
            });
            target = null;
            if (current)
                current.destroy();
        }
    }

    BlendTransitionEffect {
        id : effect

        anchors.fill : parent
        z : 0
        visible : mask === null
    }

    OpacityMask {
        anchors.fill : effect
        source : effect
        maskSource : mask
        visible : mask !== null
    }

    editor : ImageConfigurationEditor {
        id : configEditor

        anchors.fill : parent
        editor : imageBox
        configuration : p_.editConfiguration
        mask : imageBox.mask
    }
}
