//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.2

import wesual.Showcase 1.0
import wesual.Controls 1.0
import wesual.data.Core 1.0

Item {
    id : imageCarousel

    property ImageListConfiguration imageList

    property bool transitionReversed : false

    function addItem(item) {
        if (item.index < 0)
            item.index = p_.items.length;
        p_.items.push(item);
        p_.items.sort(function(i1, i2) {
            return (i1.index - i2.index);
        });
    }

    function navigateForward() {
        imageList.navigateForward();
    }

    function navigateBack() {
        imageList.navigateBack();
    }

    function prefetch() {
        for (var i = 0; i < imageList.images.length; ++i) {
            imageList.images[i].prefetch(768, 768);
        }
    }

    onImageListChanged : p_.update();

    onEnabledChanged : {
        if (enabled) {
            prefetch();
        }
    }

    Connections {
        target : imageList
        onImagesChanged       : p_.update()
        onCurrentIndexChanged : p_.update()
    }

    Component.onCompleted : {
        for (var i = 0; i < p_.items.length; ++i) {
            // Reparent item
            p_.items[i].parent = this;
            p_.items[i].reverse = Qt.binding(function() {
                return p_.reverse;
            });

            p_.items[i].onIsActiveChanged.connect(p_.updateActiveEditor);
            p_.items[i].onReady.connect(p_.startTransition);
        }

        p_.ready = true;
        p_.update();
    }

    Timer {
        id : updateTimer

        interval    : 30
        running     : false
        repeat      : false
        onTriggered : p_.scheduledUpdate()
    }

    QtObject {
        id : p_

        property bool reverse    : imageCarousel.transitionReversed
        property int  progress   : 0
        property int  baseWidth  : 0
        property int  baseHeight : 0

        property var items : []
        property var transitionList : []
        property var activeItem : null

        property int  pending;
        property bool ready : false;

        function updateActiveEditor() {
            var active = null;
            for (var i = 0; i < items.length; ++i) {
                if (items[i].isActive) {
                    active = items[i];
                    break;
                }
            }
            activeItem = active;
        }

        function startTransition(item) {
            transitionList.push(item);
            pending = Math.max(0, pending - 1);

            if (pending === 0) {
                for (var i = 0; i < transitionList.length; ++i) {
                    transitionList[i].startTransition();
                }
            }
        }

        function update() {
            if (!p_.ready)
                return;

            updateTimer.restart();
        }

        function scheduledUpdate() {
            var configurations = imageList.images;
            var len = Math.min(items.length, configurations.length);

            for (var i = 0; i < transitionList.length; ++i) {
                transitionList[i] = null;
            }

            pending = 0;
            transitionList = [];
            var index = imageList.currentIndex;
            for (i = 0; i < len; ++i) {
                var conf = configurations[(index + i) % configurations.length];
                if (p_.items[i].currentConfiguration !== conf) {
                    p_.items[i].currentConfiguration = conf;
                    pending++;
                }
            }

            for (i = len; i < items.length; ++i) {
                p_.items[i].currentConfiguration = null;
            }
        }
    }
}
