//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.4

import wesual.Showcase 1.0
import wesual.Controls 1.0
import wesual.ImageFx  1.0
import wesual.Stage   1.0

import wesual.data.Core 1.0

InlineEditor {
    id : carouselItem

    property int  index         : -1
    property bool reverse       : false
    property real progress      : 0

    property Component transitionEffect : BlendTransitionEffect {}

    readonly property ImageCarousel carousel : parent

    property ImageConfiguration currentConfiguration

    signal ready(var item)

    function startTransition() {
        transitionAnimation.start();
    }

    function finishTransition() {
        var c = p_.effect.first;

        p_.effect.first = p_.effect.second;
        carouselItem.progress = 0;
        p_.effect.second = null;
        if (c && c !== p_.effect.first) {
            c.destroy();
        }
        p_.editConfiguration = carouselItem.currentConfiguration;
    }

    width  : 100
    height : 100

    Component.onCompleted : {
        parent.addItem(this);

        var effect = transitionEffect.createObject(carouselItem);
        effect.anchors.fill = carouselItem;
        p_.effect = effect;

        p_.effect.reversed = Qt.binding(function() {
            return carouselItem.reverse;
        });
        p_.effect.progress = Qt.binding(function() {
            return carouselItem.progress;
        });
    }

    onCurrentConfigurationChanged : {
        if (p_.effect.first) {
            p_.effect.first.freeze();
        }
        p_.effect.second = buffer.createObject(carouselItem);
        p_.currentConfiguration = currentConfiguration;
        carouselItem.ready(carouselItem);
    }

    QtObject {
        id : p_

        property Item               effect
        property ImageConfiguration currentConfiguration
        property ImageConfiguration editConfiguration
    }

    ImageParameters {
        id : params

        configuration : p_.currentConfiguration
        boxWidth      : carouselItem.width
        boxHeight     : carouselItem.height
    }

    SharedImage {
        id : sharedImage

        source   : params.image
        visible  : false
        sizeHint : Math.max(params.imageWidth, params.imageHeight)
    }

    Component {
        id : buffer

        PixelBuffer {
            readonly property ImageEffectParameters effectParams :
                carouselItem.currentConfiguration.asset.imageEffectParameters

            width      : carouselItem.width
            height     : carouselItem.height
            sourceItem : sharedImage
            sourceRect : params.subrect
            z          : -1
            x          : -10000

            referenceSize {
                width  : sharedImage.nativeWidth
                height : sharedImage.nativeHeight
            }

            DefaultImageEffects {
                id : effects

                exposure        : effectParams.exposure
                lightTones      : effectParams.lightTones
                darkTones       : effectParams.darkTones
                clarity         : effectParams.clarity
                filterIntensity : effectParams.filterIntensity
                filterId        : effectParams.filterId
                vignette        : effectParams.vignette
                softLight       : effectParams.softLight
                red             : effectParams.red
                orange          : effectParams.orange
                yellow          : effectParams.yellow
                green           : effectParams.green
                aqua            : effectParams.aqua
                blue            : effectParams.blue
                purple          : effectParams.purple
                magenta         : effectParams.magenta
            }

            imageEffects : effects.imageEffects
        }
    }

    SequentialAnimation {
        id : transitionAnimation

        NumberAnimation {
            target : carouselItem
            property : "progress"
            from : 0
            to : 1
            duration : 600
            easing.type : Easing.InOutSine
        }
        ScriptAction {
            script : { carouselItem.finishTransition(); }
        }
    }

    editor : Item {
        anchors.fill : parent

        DropZone {
            anchors.fill : parent

            DocumentDropHandler {
                type : "configuration:image"
                onEntered : configEditor.forceFrame = true
                onExited  : configEditor.forceFrame = false
                onDropped : {
                    configEditor.forceFrame = false;
                    if (p_.currentConfiguration) {
                        var conf = p_.currentConfiguration;
                        conf.beginRecordChanges();
                        try {
                            var configs = drag.source.dragDocuments;
                            var currentIdx = carousel.imageList.currentIndex;
                            if (configs.length > 1) {
                                for (var i = configs.length-1; i > 0; --i) {
                                    var imgConf = configs[i];
                                    carousel.imageList.moveAsset(
                                            carousel.imageList.indexOf(imgConf),
                                            carousel.imageList.indexOf(conf));
                                }
                            }
                            carousel.imageList.swap(configs[0], conf);
                            carousel.imageList.currentIndex = currentIdx;
                        } catch (e) {}
                        conf.endRecordChanges();
                    }
                }
            }
        }

        ImageConfigurationEditor {
            id : configEditor

            anchors.fill  : parent
            editor        : carouselItem
            configuration : p_.editConfiguration
            forceHidden   : transitionAnimation.running
        }
    }
}
