//
// (c) 2020 wesolutions GmbH
// All rights reserved
//

import QtQuick 2.3

import wesual.Showcase 1.0
import wesual.Controls 1.0

import wesual.data.Core 1.0

InlineEditor {
    id : imageGallery

    property ImageListConfiguration imageList

    property Component transitionEffect : BlendTransitionEffect {}
    property alias transitionDuration   : fadeAnimation.duration
    property alias transitionEasing     : fadeAnimation.easing

    function navigateBack() {
        imageList.navigateBack();
    }

    function navigateForward() {
        imageList.navigateForward();
    }

    enabled : imageList.currentImage

    onVisibleChanged : {
        if (visible) {
            try { imageList.prefetch(2, width, height); } catch (e) {}
        }
    }

    Component.onCompleted : {
        var effect = transitionEffect.createObject(imageGallery);
        p_.effect = effect;
        effect.anchors.fill = imageGallery;
        effect.z = 0;

        p_.currentImage = imageList.currentImage;
    }

    Connections {
        target : imageList
        ignoreUnknownSignals : true
        onCurrentImageChanged : {
            p_.currentImage = imageList.currentImage;
            if (imageGallery.visible) {
                try { imageList.prefetch(2, width, height); } catch (e) {}
            }
        }
    }

    QtObject {
        id : p_

        property bool reverseTransition : false

        property ImageConfiguration currentImage
        property ImageConfiguration editConfiguration
        property Item effect

        property var first  : null
        property var second : null
        property var next   : null

        onCurrentImageChanged : {
            if (next)
                next.discard();
            next = galleryEntry.createObject(imageGallery, {
                "configuration" : imageList.currentImage
            });
            next.instanceTexture();
        }
    }

    Component {
        id : textureTemplate

        TextureSource { mipmap : false }
    }

    Component {
        id : galleryEntry

        ImageParameters {
            property TextureSource texture

            signal textureReady()

            boxWidth  : imageGallery.width
            boxHeight : imageGallery.height

            Binding {
               when : texture !== null
               target : texture

               property : "requiredSize"
               value : Qt.size(imageWidth, imageHeight)
            }
            Binding {
                when : texture !== null
                target : texture
                property : "url"
                value : image
            }
            Connections {
                target : texture
                onTextureReady : textureReady()
            }

            function instanceTexture() {
                if (texture && texture.ready) {
                    textureReady();
                } else {
                    texture = textureTemplate.createObject(imageGallery);
                }
            }

            function discardTexture() {
                if (texture)
                    texture.destroy();
                texture = null;
            }

            function discard() {
                discardTexture();
                destroy();
            }
        }
    }

    Binding {
        target : p_.effect
        property : "first"
        value : p_.first ? p_.first.texture : null
    }

    Binding {
        target : p_.effect
        property : "firstSubrect"
        value : p_.first ? p_.first.subrect : Qt.rect(0, 0, 1, 1)
    }

    Binding {
        target : p_.effect
        property : "second"
        value : p_.second ? p_.second.texture : null
    }

    Binding {
        target : p_.effect
        property : "secondSubrect"
        value : p_.second ? p_.second.subrect : Qt.rect(0, 0, 1, 1)
    }

    Connections {
        id : transitionLauncher
        target : p_.next
        ignoreUnknownSignals : true
        onTextureReady : {
            p_.editConfiguration = imageGallery.imageList.currentImage;

            if (imageGallery.visible) {
                var c = p_.first;
                p_.first = p_.second;
                p_.second = p_.next;
                p_.next = c;
                fadeAnimation.start();
            } else {
                c = p_.first;
                p_.first = p_.next;
                p_.next = null;
                if (c)
                    c.discard();
            }
        }
    }

    NumberAnimation {
        id : fadeAnimation

        target : p_.effect
        property : "progress"
        from : 0.0
        to : 1.0
        duration : transitionDuration
    }

    editor : ImageConfigurationEditor {
        id : configEditor
        anchors.fill : parent
        configuration : p_.editConfiguration
        editor : imageGallery
        forceHidden : fadeAnimation.running
    }
}
