//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.2

import wesual.Showcase 1.0
import wesual.Controls 1.0

Item {
    id : scaleSlider

    property Item attachedItem
    property real maxValue      : 2.0

    readonly property bool isPeaked : p_.isPeaked
    readonly property real parentWidth : attachedItem.width / counterScale

    signal scaleChanged(real scale)

    function setScale(scale) {
        p_.externalScale = scale;
    }

    function show() {
        popup.show();
    }

    function hide() {
        popup.hide();
    }

    anchors.horizontalCenter : attachedItem.horizontalCenter
    anchors.bottom : attachedItem.bottom

    QtObject {
        id : p_

        property real activeOpacity         : 0.95
        property real deactivatedOpacity    : 0.5
        property real externalScale         : 1.0
        property bool isPeaked              : false
    }

    PopupAnchorItem {
        id : popup

        onPopupItemChanged : {
            if (popupItem) {
                p_.isPeaked = (parentWidth > 1.5 * popupItem.width) ? false
                                                                    : true;
            }
        }
        anchors.centerIn : parent

        persistent : true

        popup : Item {

            readonly property bool positionOnTop : popup.distanceBottom < 50
            readonly property bool bottomUp : scaleSlider.isPeaked
                                              && positionOnTop

            // update slidervalue after creation
            Connections {
                target : p_
                ignoreUnknownSignals : true
                onExternalScaleChanged : {
                    slider.value = p_.externalScale;
                }
            }

            width : bg.width
            height : 50

            opacity : popup.popupVisible ? 1 : 0
            Behavior on opacity {
                NumberAnimation { duration : 150 }
            }

            anchors {
                horizontalCenter : parent.horizontalCenter
                bottom : parent.bottom
                bottomMargin : {
                    if (scaleSlider.isPeaked && !positionOnTop)
                        return -(bg.height+6); // outside

                    if (bottomUp)
                        return attachedItem.height/counterScale+6; // above

                    return 6; // in item
                }
            }

            Image {
                id : bg
                source : scaleSlider.isPeaked
                         ? "qrc:/showcase/images/zoom-overlay-pointer.png"
                         : "qrc:/showcase/images/zoom-overlay-rect.png"
                opacity : p_.deactivatedOpacity

                Behavior on opacity {
                    NumberAnimation { duration : 150 }
                }
                rotation : bottomUp ? 180 : 0
                transformOrigin : Item.Center
            }

            Item {
                id : wrapper
                width : bg.width - 2 * 6
                height : 30


                anchors {
                    horizontalCenter : bg.horizontalCenter
                    top : bottomUp ? bg.top : undefined
                    topMargin : bottomUp ? 10 : 0
                    bottom : !scaleSlider.isPeaked || !positionOnTop ? bg.bottom
                                                                     : undefined
                    bottomMargin : !positionOnTop || !scaleSlider.isPeaked
                                   ? (scaleSlider.isPeaked ? 10 : 7) : 0
                }

                opacity : p_.deactivatedOpacity

                Behavior on opacity {
                    NumberAnimation { duration : 150 }
                }

                Image {
                    id : zoomOut
                    source : "qrc:/showcase/icons/zoom-out.png"
                    anchors.left : wrapper.left
                    anchors.leftMargin : 5
                    anchors.top : wrapper.top
                    anchors.topMargin :
                        Math.floor((wrapper.height - zoomOut.height) / 2)
                }

                Image {
                    id : zoomIn
                    source : "qrc:/showcase/icons/zoom-in.png"
                    anchors.left : slider.right
                    anchors.leftMargin : 5
                    anchors.top : wrapper.top
                    anchors.topMargin :
                        Math.floor((wrapper.height - zoomIn.height) / 2);
                }

                Slider {
                    id : slider

                    width : wrapper.width - zoomIn.width - zoomOut.width - 20
                    height : 15
                    anchors {
                        left : zoomOut.right
                        leftMargin : 5
                        top : wrapper.top
                        topMargin :
                            Math.floor((wrapper.height - slider.height) / 2)
                    }
                    slidePathHeight : 1
                    slideControlWidth : 3
                    slideControlHeight : 15

                    //value : 1.0
                    minimum : 1.0
                    maximum : scaleSlider.maxValue
                    onValueChanged : {
                        scaleSlider.scaleChanged(value)
                    }
                    hoverColor : UI.color(UI.PrimaryPress)
                    Component.onCompleted : slider.value = p_.externalScale
                }
            } // wrapper
            MouseArea {
                id : sliderWrapperMouse
                anchors.fill : parent
                hoverEnabled : true

                function overItem(mouse, item) {
                    var mappedItem = item.mapToItem(sliderWrapperMouse, 0, 0);

                    return (mouse.x >= mappedItem.x &&
                            mouse.x <= (mappedItem.x + item.width) &&
                            mouse.y >= mappedItem.y &&
                            mouse.y <= (mappedItem.y + item.height));
                }

                onPositionChanged : {
                    // set hover state manually because of overlapping mouse areas
                    if (overItem(mouse, slider))
                        slider.state = "hovered";
                    else
                        slider.state = "";

                    if (overItem(mouse, zoomIn) || overItem(mouse, zoomOut)) {
                        sliderWrapperMouse.cursorShape = Qt.PointingHandCursor;
                    } else
                        sliderWrapperMouse.cursorShape = Qt.ArrowCursor;

                    // onPositionChanged is handled in slider control
                    mouse.accepted = false;
                }
                onEntered : {
                    bg.opacity = p_.activeOpacity;
                    wrapper.opacity = p_.activeOpacity;
                }
                onExited : {
                    bg.opacity = p_.deactivatedOpacity;
                    wrapper.opacity = p_.deactivatedOpacity;
                }
                onPressed : {
                    // press is handled by slider control
                    if (overItem(mouse, slider))
                        mouse.accepted = false;
                }
                onClicked : {
                    if (overItem(mouse, zoomIn))
                        slider.value = slider.maximum;

                    if (overItem(mouse, zoomOut))
                        slider.value = slider.minimum;
                }
            }
        }
    }
}
