//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.2

import wesual.Showcase 1.0

Item {
    id : instantiator

    readonly property Item currentItem : p_.currentItem

    property Component delegate
    property Component transitionEffect : defaultTransition

    property alias transitionDuration : transitionAnimation.duration
    property alias transitionEasing   : transitionAnimation.easing
    property bool  transitionReversed : false

    function instanceNext(data) {
        if (!delegate)
            return;

        if (instantiator.visible) {
            transitionContainer.visible = true;
            p_.nextItem = delegate.createObject(instantiator, data || {});
            p_.nextItem.visible = false;
            p_.nextItem.anchors.fill = instantiator;
            sourceCurrent.scheduleUpdate();
            transitionAnimation.start();
        } else {
            if (p_.currentItem)
                p_.currentItem.destroy();
            p_.currentItem = delegate.createObject(instantiator, data || {});
            p_.currentItem.anchors.fill = instantiator;
        }
    }

    onTransitionEffectChanged : p_.createTransitionEffect()
    Component.onCompleted     : p_.createTransitionEffect()

    QtObject {
        id : p_

        property Item effect
        property Item nextItem
        property Item currentItem

        function createTransitionEffect() {
            if (effect) {
                effect.destroy();
                effect = null;
            }
            effect = instantiator.transitionEffect.createObject(
                        transitionContainer);
            if (effect) {
                effect.z = 1;
                effect.anchors.fill = transitionContainer;
                effect.first = sourceCurrent;
                effect.second = sourceNext;
            }
        }
    }

    Component {
        id : defaultTransition

        BlendTransitionEffect {}
    }

    Binding {
        target   : p_.currentItem
        when     : p_.currentItem
        property : "z"
        value    : 2
    }

    Binding {
        target   : p_.nextItem
        when     : p_.nextItem
        property : "z"
        value    : 1
    }

    Binding {
        target   : p_.effect
        when     : p_.effect
        property : "reversed"
        value    : instantiator.transitionReversed
    }

    Item {
        id : transitionContainer

        anchors.fill : parent
        z : 3
        visible : false

        ShaderEffectSource {
            id : sourceCurrent

            width : parent.width
            height : parent.height
            live : false
            mipmap : false
            antialiasing : false
            recursive : true
            z : -1
            sourceItem : p_.currentItem
        }

        ShaderEffectSource {
            id : sourceNext

            width : parent.width
            height : parent.height
            live : true
            mipmap : false
            antialiasing : false
            recursive : true
            z : -2
            sourceItem : p_.nextItem
        }

        NumberAnimation {
            id : transitionAnimation

            target : p_.effect
            duration : 500
            property : "progress"
            from : 0
            to : 1
            easing.type : Easing.InOutQuad
            running : false
            alwaysRunToEnd : true
            onStopped : {
                if (p_.currentItem) {
                    p_.currentItem.destroy();
                }
                p_.currentItem = p_.nextItem;
                p_.nextItem = null;
                p_.currentItem.visible = true;
                transitionContainer.visible = false;
                p_.effect.progress = 0;
            }
        }
    }
}
