//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.3
import QtGraphicalEffects 1.0

import QtQuick.Layouts  1.1

import wesual.Controls 1.0
import wesual.Showcase 1.0

import wesual.data.Core 1.0

FocusScope {
    id : logoPicker

    property list<LogoItem> logos

    property IntegerConfiguration logoIndex

    property int logoWidth : 60
    property int logoHeight : 40

    property int currentIndex : logoIndex.value

    readonly property bool open : popup.popupVisible
    readonly property var currentLogo : {
        if (currentIndex >= 0 && currentIndex < logos.length) {
            return logos[currentIndex];
        }
        return logos[0];
    }

    function openPopup()  {
        popup.activeIndex = currentIndex;
        popup.show();
    }

    width : 160
    height : 106

    Rectangle {
        id : container

        width : parent.width
        height : parent.height

        color : UI.color(UI.PrimaryBase)

        border {
            width : 1
            color : {
                if (!logoPicker.enabled)
                    return UI.color(UI.DisabledBorder);

                if (topMouseArea.pressed || logoPicker.open) {
                    return UI.color(UI.ControlBorderPress);
                } else if (topMouseArea.containsMouse ||
                           logoPicker.activeFocus) {
                    return UI.color(UI.ControlBorderHover);
                }

                return "transparent"
            }
        }
        Behavior on border.color {
            ColorAnimation { duration : 100 }
        }

        Image {
            id : backgroundImage

            anchors {
                left : parent.left
                leftMargin : 6
                top : parent.top
                topMargin : 6
            }

            width : 132
            height : 94

            source : currentLogo.backgroundImage.source

            fillMode : Image.PreserveAspectFit
        }
        Image {
            id : selectedImage

            anchors.fill : backgroundImage
            anchors.margins : 6

            source : currentLogo.image.source

            fillMode : Image.PreserveAspectFit
        }

        ColorizedImage {
            id : indicator

            anchors {
                right : parent.right
                rightMargin : 6
                verticalCenter : parent.verticalCenter
                verticalCenterOffset : 1
            }

            source : "qrc:/controls/icons/dropdown-indicator.png"

            color : UI.color(UI.DropdownIndicatorIcon)
        }
    }

    MouseArea {
        id : topMouseArea

        hoverEnabled : true
        anchors.fill : container
        acceptedButtons : Qt.LeftButton
        cursorShape : Qt.PointingHandCursor
        onPressed : logoPicker.open ? popup.hide() : logoPicker.openPopup()
    }

    LogoPickerPopup {
        id : popup

        items       : logoPicker.logos
        logoIndex   : logoPicker.logoIndex
        activeIndex : logoPicker.currentIndex
    }
}
