//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.3

import QtQuick.Layouts  1.1

import wesual.Controls 1.0
import wesual.Showcase 1.0

import wesual.data.Core        1.0

PopupAnchorItem {
    id : popup

    property var items
    property int activeIndex : -1

    property IntegerConfiguration logoIndex

    property LogoItem selectedImage : items[logoIndex.value]

    signal accepted(int result)

    anchors.left : parent.left
    anchors.top : parent.bottom

    width : childrenRect.width
    height : childrenRect.height

    onAccepted : {
        logoIndex.beginRecordChanges();
        logoIndex.value = result;
        logoIndex.endRecordChanges();
        popup.hide();
    }

    Keys.onEscapePressed : popup.hide()

    popup : Rectangle {
        id : popupItem

        opacity : 0

//        width  : content.implicitWidth + 19
//        height : {
//            if (scrollbarsVisible) {
//                return 600;
//            } else {
//                return content.implicitHeight;
//            }
//        }

        width : scrollbarsVisible ? 310 : 294
        height : preferredHeight
        border {
            color : UI.color(UI.PrimaryControlBorder)
            width : 1
        }

        readonly property int  preferredHeight   :
            Math.min(Math.floor(popup.items.length / 2) * 106 + 15, 330)
        readonly property bool scrollbarsVisible : content.implicitHeight > popupItem.height
        readonly property bool shift : popup.distanceRight < width

        anchors {
            left : parent.left
            leftMargin : shift ? popup.parent.width - width : 0
            top : parent.bottom
            topMargin : -1
        }

        states : [
            State {
                name : "visible"
                when : popup.popupVisible

                PropertyChanges {
                    target : popupItem
                    opacity : 1
                }
            }
        ]

        transitions : Transition {
            NumberAnimation {
                property : "opacity"
                duration : 200
            }
        }

        MouseArea { // catch Mouse on background to avoid popup close
            id : catchArea

            anchors.fill : parent
            onClicked : {}
        }

        ScrollView {
            id : scrollView
            width : parent.width - 2
            height : parent.height - 10
            anchors.centerIn : parent

            GridLayout {
                id : content

                columns : 2

                rowSpacing : 0.0
                columnSpacing : 0.0

                width : scrollView.viewport.width

                Repeater {
                    model : popup.items

                    delegate : Rectangle {
                        id : delegate

                        visible : modelData !== selectedImage

                        width : background.width + 12
                        height : background.height + 12

                        color : mouseArea.pressed ? UI.color(UI.PrimaryPress)
                                                  : (mouseArea.containsMouse
                                                     ? UI.color(UI.PrimaryHover)
                                                     : UI.color(UI.PrimaryBase))
                        Behavior on color {
                            ColorAnimation { duration: 200 }
                        }

                        Image {
                            id : background

                            anchors {
                                left : parent.left
                                leftMargin : 6
                                top : parent.top
                                topMargin : 6
                            }

                            width : 132
                            height : 94

                            source : modelData.backgroundImage.source

                            fillMode : Image.PreserveAspectFit
                        }
                        Image {
                            id : icon

                            anchors.fill : background
                            anchors.margins : 6

                            source : modelData.image.source

                            fillMode : Image.PreserveAspectFit
                        }

                        MouseArea {
                            id : mouseArea

                            anchors.fill : delegate
                            hoverEnabled : true

                            acceptedButtons : Qt.LeftButton
                            cursorShape : Qt.PointingHandCursor

                            onClicked : {
                                popup.accepted(index);
                            }
                        }
                    } // repeater delegate
                }
            } // column
        }
    } // contentItem
} // popup
