//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick            2.4
import QtMultimedia       5.4

import wesual.Ui       1.0

Rectangle {
    id : background

    property MediaPlayer player
    property bool        display       : false
    property bool        containsMouse : mouseArea.containsMouse
    property int         duration      : 500

    width   : 34
    height  : width
    color   : UiColors.getColor(UiColors.VideoButtonColor)
    opacity : 0.6
    visible : display

    states : [
        State {
            name : "pressed"
            when : mouseArea.pressed
            PropertyChanges {
                target  : background
                color   : UiColors.getColor(UiColors.ButtonPressBlue)
                opacity : 1.0
            }
        },
        State {
            name : "hovered"
            when : mouseArea.containsMouse
            PropertyChanges {
                target  : background
                color   : UiColors.getColor(UiColors.ButtonHoverBlue)
                opacity : 0.95
            }
        }
    ]

    transitions : Transition {
        PropertyAnimation {
            target     : background
            properties : "opacity"
            duration   : 150
        }
        ColorAnimation {
            duration : 100
        }
    }

    UiIcon {
        id : icon2

        iconName : "video-pause"
        anchors.centerIn : parent
    }

    UiIcon {
        id : icon

        iconName : "video-play"
        anchors.centerIn : parent

        states : [
            State {
                name : "swap"
                when : player &&
                       player.playbackState === MediaPlayer.PlayingState
            },
            State {
                name : "swapBack"
                when : player &&
                       player.playbackState !== MediaPlayer.PlayingState
            }
        ]
        transitions : [
            Transition {
                to : "swap"
                NumberAnimation {
                    target      : icon
                    properties  : "opacity, scale"
                    easing.type : Easing.InOutQuad
                    duration    : duration
                    to          : 0.1
                }
                NumberAnimation {
                    target      : icon2
                    properties  : "opacity, scale"
                    easing.type : Easing.InOutQuad
                    duration    : duration
                    to          : 1
                }
            },
            Transition {
                to : "swapBack"
                NumberAnimation {
                    target      : icon2
                    properties  : "opacity, scale"
                    easing.type : Easing.InOutQuad
                    duration    : duration
                    to          : 0.1
                }
                NumberAnimation {
                    target      : icon
                    properties  : "opacity, scale"
                    easing.type : Easing.InOutQuad
                    duration    : duration
                    to          : 1
                }
            }
        ]
    }

    MouseArea {
        id : mouseArea

        anchors.fill    : parent
        hoverEnabled    : true
        cursorShape     : Qt.PointingHandCursor

        onClicked : {
            if (!player)
                return;

            if (player.playbackState === MediaPlayer.PlayingState)
                player.pause();
            else
                player.play();
        }
    }
}
