//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick      2.5
import QtMultimedia 5.5

import wesual.Ui 1.0

Item {
    id : progressBar

    property MediaPlayer player
    property bool        display       : false
    property bool        containsMouse : mouseArea.containsMouse
    property int         barWidth      : 50
    property int         barHeight     : 11
    property int         barMargin     : 2

    // 0 .. 1 of total video length
    property real progress : player && player.duration > 0
                             ? player.position / player.duration : 0.0

    implicitWidth  : barWidth
    implicitHeight : barHeight
    visible        : display

    Rectangle {
        id : progressWrapper

        anchors.fill : parent
        color        : UiColors.getColor(UiColors.VideoButtonColor)
        opacity      : 0.6

        states : [
            State {
                name : "pressed"
                when : mouseArea.pressed

                PropertyChanges {
                    target  : progressWrapper
                    color   : UiColors.getColor(UiColors.ButtonPressBlue)
                    opacity : 1.0
                }
            },
            State {
                name : "hovered"
                when : mouseArea.containsMouse

                PropertyChanges {
                    target  : progressWrapper
                    color   : UiColors.getColor(UiColors.ButtonHoverBlue)
                    opacity : 0.95
                }
            }
        ]
        transitions : Transition {
            PropertyAnimation {
                target     : progressWrapper
                properties : "opacity"
                duration   : 150
            }
            ColorAnimation {
                duration : 100
            }
        }
    }
    Rectangle {
        width : player ? parent.width * progress : 0
        color : UiColors.getColor(UiColors.White)
        anchors {
            left       : parent.left
            leftMargin : barMargin
            top        : parent.top
            bottom     : parent.bottom
            margins    : barMargin
        }

        Behavior on width { NumberAnimation { duration : 150 } }
    }

    MouseArea {
        id : mouseArea

        cursorShape  : Qt.PointingHandCursor
        hoverEnabled : true
        anchors {
            fill        : parent
            leftMargin  : barMargin
            rightMargin : barMargin
        }
        onClicked : {
            if (!player) {
                return;
            }

            var vid = player;
            var seekPercentage = mouse.x / mouseArea.width;
            vid.seek(parseInt(vid.duration * seekPercentage));
        }
    }
}
