//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.2

import wesual.Controls 1.0
import wesual.Showcase 1.0

Rectangle {
    property Draggable          draggable
    property StateConfiguration configuration
    property string             targetState
    property int                index : 0
    property bool               circular : false

    property int indexPixelSize : dropZone.containsDrag ? 35 : 19

    clip : true

    width : 100
    height : 100
    radius : circular ? parseInt(width / 2) : 0

    color : dropZone.containsDrag ? "#AA000000" : "#33000000"
    opacity : draggable && draggable.isDragged ? 1 : 0

    border.color : "#33FFFFFF"
    border.width : dropZone.containsDrag ? 0 : parseInt(2 * counterScale)

    Behavior on opacity {
        NumberAnimation { duration : 200 }
    }
    Behavior on color {
        ColorAnimation {duration : 200 }
    }
    Behavior on border.width {
        NumberAnimation { duration : 100 }
    }    
    Behavior on indexPixelSize {
        NumberAnimation { duration : 200 }
    }

    Item {
        width : parent.width - 2 * parent.border.width
        height : parent.height - 2 * parent.border.width

        anchors {
            left : parent.left
            leftMargin : parent.border.width
            top : parent.top
            topMargin : parent.border.width
        }

        clip : true

        Rectangle {
            width : dropZone.containsDrag ?
                        parseInt(parent.width * 0.66 / counterScale) : 42
            height : width
            radius : parseInt(width / 2)
            color : UI.color(UI.TextSelection)
            opacity : 0.85

            anchors.centerIn : parent

            scale : counterScale || 1.0

            Behavior on width {
                NumberAnimation { duration : 200 }
            }

            Text {
                anchors.centerIn : parent
                text : index

                font.family : UI.fontFamily(UI.PrimaryFont)
                font.weight : UI.fontWeight(UI.PrimaryFont)
                font.pixelSize : indexPixelSize

                color : "#FFFFFF"
            }
        }
    }


    DropZone {
        id : dropZone

        anchors.fill : parent

        DraggableDropHandler {
            key : draggable ? draggable.dragData : ""
            onDropped : {
                if (!configuration)
                    return;

                configuration.beginRecordChanges();
                configuration.state = targetState;
                configuration.endRecordChanges();
            }
        }
    }
}
