//
// (c) 2020 wesolutions GmbH
// All rights reserved
//

import QtQuick 2.2

import wesual.Showcase 1.0
import wesual.Controls 1.0

import wesual.data.Core 1.0

InlineEditor {
    id : priceBox

    property Product product

    property alias font                 : text.font
    property alias horizontalAlignment  : text.horizontalAlignment
    property alias color                : text.color
    property bool  showCurrencyLabel    : true
    property int   currencyLabelSpacing : text.font.pixelSize / 2

    property real price : {
        if (!product)
            return -1;

        var country = product.context.country;
        return product.getPrice(country);
    }

    Connections {
        target : product ? product : null
        onPricesChanged : {
            var country = product.context.country;
            priceBox.price = product.getPrice(country);
        }
    }

    readonly property string formattedPrice : {
        if (!product)
            return "";

        var locale = Qt.locale(product.language);

        return price.toLocaleString(locale);
    }

    readonly property string currencyString : {
        if (!product || !showCurrencyLabel)
            return "";

        return UI.currencyString(
                    product.context.country, text.font.family);
    }

    implicitWidth  : texts.implicitWidth
    implicitHeight : texts.implicitHeight
    baselineOffset : text.baselineOffset

    Row {
        id : texts

        spacing : currencyLabelSpacing
        height  : text.implicitHeight
        width   : childrenRect.width

        Text {
            font : text.font
            color : text.color
            text : currencyString
            visible : showCurrencyLabel
            anchors.verticalCenter : text.verticalCenter
        }

        PriceEditor {
            id : text
            background : Item {}
            cursorDelegate : cursorDelegateComponent
            width : implicitWidth
            wrapMode : Text.NoWrap

            document : product

            onActiveFocusChanged : {
                priceBox.makeActive();
            }
        }
    }

    Component {
        id : cursorDelegateComponent

        Rectangle {
            id : cursor

            width : 2 * counterScale
            color : text ? text.color : "black"

            // Cursor blink timer
            Timer {
                running : true
                interval : 600
                repeat : true
                onTriggered : cursor.opacity = Math.abs(cursor.opacity - 1.0)
            }
        }
    }

    editor : Frame {
        anchors.fill : parent

        color : text.color

        opacity : {
            if (priceBox.isActive && text.activeFocus) {
                return 1;
            } else if (priceBox.containsMouse) {
                return .5;
            } else {
                return 0;
            }
        }
    }
}
