//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.3
import QtQuick.Layouts  1.1

import wesual.Controls          1.0
import wesual.Showcase          1.0
import wesual.data.Core         1.0
import wesual.module.MediaManager      1.0

SidebarItem {
    id : editor

    property ImageConfiguration imageConfiguration : null

    width  : parent ? parent.width : 100
    height : content.implicitHeight

    readonly property string socialShareQml : "
        import QtQuick 2.3

        import wesual.Editor 1.0

        SocialShareButton {
            id : shareButton
            documents   : imageConfiguration ? [ imageConfiguration.asset ] : null
            anchors.left : picker.right
            anchors.leftMargin : 10
        }"

    signal navigateToProduct()

    // prevent deselection of the current product on click outside of controls
    MouseArea {
        id : stealClickFromEditor

        width  : content.width
        height : content.height
        onClicked : {}
    }

    Column {
        id : content

        property ImageAsset image : imageConfiguration ?
                                    imageConfiguration.asset : null

        width  : parent.width
        height : childrenRect.height
        anchors.horizontalCenter : parent.horizontalCenter
        spacing :  9

        Item {
            id : imageWrapper

            width  : 110
            height : 110

            ImagePickerButton {
                id : picker
                width : 110
                height : 110
                imageSource : imageConfiguration && imageConfiguration.asset ?
                              imageConfiguration.asset.file : ""
                document : imageConfiguration ?
                           imageConfiguration.document : null
                onSelectedImages : {
                    imageConfiguration.document.beginRecordChanges();
                    imageConfiguration.document = images[0];
                    imageConfiguration.document.endRecordChanges();
                }

                onNavigateToProduct : {
                    editor.navigateToProduct();
                }
            }

            Component.onCompleted :
                Qt.createQmlObject(socialShareQml, imageWrapper)
        }
        GridLayout {
            columns : 2
            columnSpacing : 19
            rows : 3

            Label {
                text : qsTrId("label.Name")
                font.pixelSize : 11
                Layout.alignment : Qt.AlignBottom
                lineHeightMode : Text.FixedHeight
                lineHeight : 14
            }
            Text {
                text : content.image ? content.image.label : ""
                font {
                    family : UI.fontFamily(UI.PrimaryFont)
                    weight : UI.fontWeight(UI.PrimaryFont)
                    pixelSize : 14
                }
                Layout.fillWidth : true
                color : UI.color(UI.SecondaryTextColor)
                maximumLineCount : 1
                elide : Qt.ElideRight
                wrapMode : Text.Wrap
                lineHeightMode : Text.FixedHeight
                lineHeight: 17
            }
            Label {
                text : qsTrId("label.Media.Info.Type")
                font.pixelSize : 11
                Layout.alignment : Qt.AlignBottom
                lineHeightMode : Text.FixedHeight
                lineHeight : 14
            }
            Text {
                text : content.image ?
                       content.image.fileExt.toUpperCase() : ""
                font {
                    family : UI.fontFamily(UI.PrimaryFont)
                    weight : UI.fontWeight(UI.PrimaryFont)
                    pixelSize : 14
                }
                Layout.fillWidth : true
                color : UI.color(UI.SecondaryTextColor)
                maximumLineCount : 1
                elide : Qt.ElideRight
                wrapMode : Text.Wrap
                lineHeightMode : Text.FixedHeight
                lineHeight: 17
            }
            Label {
                text : qsTrId("label.Media.Info.Size")
                font.pixelSize : 11
                Layout.alignment : Qt.AlignBottom
                lineHeightMode : Text.FixedHeight
                lineHeight : 14
            }
            Text {
                text : content.image ?
                       content.image.size.width + " x " + content.image.size.height : ""
                font {
                    family : UI.fontFamily(UI.PrimaryFont)
                    weight : UI.fontWeight(UI.PrimaryFont)
                    pixelSize : 14
                }
                Layout.fillWidth : true
                color : UI.color(UI.SecondaryTextColor)
                maximumLineCount : 1
                elide : Qt.ElideRight
                wrapMode : Text.Wrap
                lineHeightMode : Text.FixedHeight
                lineHeight: 17
            }
        }
    }
}
