//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.3

import wesual.Showcase          1.0
import wesual.Controls          1.0
import wesual.data.Core         1.0
import wesual.data.FoodService  1.0

SidebarItem {
    id : editor

    property ImageListConfiguration imageList
    property ImageConfiguration     selectedImage

    property int currentPageIndex : -1

    // for ChangeImage action
//    readonly property FoodServiceProduct selectedProduct : {
//        if (selectedImage && selectedImage.document) {
//            return selectedImage.document;
//        }

//        return null;
//    }

    property int length : imageList.length

    Connections {
        target : imageList

        onLengthChanged : {
            if (imageList.length >= length) {
                gridView.selection.push(imageList.images[imageList.length - 1]);
            }

            length = imageList.length;
        }
    }

    property bool   highlightCurrent    : true
    property bool   selectableItems     : false
    property string displayPropertyName : ""

    property Item mediaDialog

    signal doubleClicked(int index)

    Component.onCompleted : {
        var comp = Qt.createComponent(
                    "qrc:/showcase/components/GlobalDialogComponent.qml");
        if(comp) {
            editor.mediaDialog = comp.createObject(editor);
        }
    }

    Connections {
        target : mediaDialog
        ignoreUnknownSignals : true
        onAccepted : {
            menu.targetItem.imageConfig.document.beginRecordChanges();
            menu.targetItem.imageConfig.document.primaryImage = result;
            menu.targetItem.imageConfig.document.endRecordChanges();
        }
    }

    height : gridView.height
    panel : SidebarItem.RightScrollablePanel
    width : parent.width - 2

    function remove(imageConfig) {
        editor.imageList.beginRecordChanges();
        editor.imageList.remove(imageConfig);
        editor.imageList.endRecordChanges();
    }

    function removeSelection() {
        var selection = gridView.selection;
        var len = gridView.selection.length;

        for (var i = 0; i < len; ++i) {
            editor.remove(selection[i]);
        }
        deselectImage();
    }

    function deselectImage() {
        gridView.deselect();
        editor.selectedImage = null;
    }

    ContextMenu {
        id : menu

        items : [
            Action {
                text : qsTrId("menu.SideBarImage.Remove")
                iconSource : "qrc:/controls/icons/remove.png"
                onTriggered : {
                    if(menu.targetItem) {
                        removeSelection();
                    }
                }
            },
//            Action {
//                text : qsTrId("menu.SideBarImage.ChangeImage")
////                    iconSource : "qrc:/showcase/icons/duplicate-content.png"
//                enabled : gridView.selectionLength === 1
//                onTriggered : {
//                    if(menu.targetItem && menu.targetItem.imageConfig
//                            && editor.mediaDialog) {
//                        editor.mediaDialog.show();
//                    }
//                }
//            },
            Action {
                text : qsTrId("menu.SideBarImage.Duplicate")
                iconSource : "qrc:/showcase/icons/duplicate-content.png"
                onTriggered : {
                    editor.imageList.beginRecordChanges();
                    var configs = gridView.selection;
                    var lastIdx = -1;
                    for (var i = 0; i < configs.length; ++i) {
                        lastIdx = Math.max(lastIdx,
                                           editor.imageList.indexOf(configs[i]))
                    }
                    var newSelection = [];
                    for (var j = gridView.selection.length-1; j >= 0 ; --j) {
                        var config = gridView.selection[j];
                        var newConf = editor.imageList.addAsset(config.document,
                                                                config.property,
                                                                lastIdx+1);
                        newSelection.push(newConf);
                    }
                    editor.imageList.endRecordChanges();

                    gridView.selection = newSelection;
                }
            },
            Action {
                text : qsTrId("menu.SideBarImage.MoveToFirstPosition")
                enabled : {
                    var imgs = editor.imageList.images;
                    if  (gridView.selection.length === 1
                            && gridView.selection[0] !== imgs[0])
                        return true;

                    var indexes = [];
                    for (var i = 0; i < gridView.selection.length; ++i) {
                        indexes.push(editor.imageList.indexOf(
                                         gridView.selection[i]));
                    }
                    indexes.sort(function(a, b){return a-b});

                    if (indexes[0] !== 0)
                        return true;

                    for (var j = 0; j < indexes.length-1; ++j ) {
                         if (indexes[j+1] - indexes[j] !== 1)
                             return true;
                    }

                    return false;
                }
                onTriggered : {
                    editor.imageList.beginRecordChanges();
                    var configs = gridView.selection;
                    var sorted = [];
                    for (var i = 0; i < configs.length; ++i) {
                          if (sorted.length === 0
                                  || editor.imageList.indexOf(sorted[i-1])
                                  > editor.imageList.indexOf(configs[i]))
                            sorted.push(configs[i]);
                         else
                            sorted = [configs[i]].concat(sorted);
                    }

                    for (var j = 0; j < sorted.length; ++j ) {
                        editor.imageList.moveAsset(
                                    editor.imageList.indexOf(sorted[j]), 0);
                    }
                    editor.imageList.endRecordChanges();
                }
            },
            Action {
                text : qsTrId("menu.SideBarImage.MoveToLastPosition")
                enabled : {
                    var imgs = editor.imageList.images;
                    if  (gridView.selection.length === 1
                            && gridView.selection[0] !== imgs[imgs.length-1])
                        return true;

                    var indexes = [];
                    for (var i = 0; i < gridView.selection.length; ++i) {
                        indexes.push(editor.imageList.indexOf(
                                         gridView.selection[i]));
                    }
                    indexes.sort(function(a, b){return b-a});

                    if (indexes[0] !== imgs.length-1)
                        return true;

                    for (var j = 0; j < indexes.length-1; ++j ) {
                         if (indexes[j] - indexes[j+1] !== 1)
                             return true;
                    }

                    return false;
                }
                onTriggered : {
                    editor.imageList.beginRecordChanges();
                    var configs = gridView.selection;
                    var sorted = [];
                    for (var i = 0; i < configs.length; ++i) {
                         if (sorted.length === 0
                                 || editor.imageList.indexOf(sorted[i-1])
                                 < editor.imageList.indexOf(configs[i]))
                            sorted.push(configs[i]);
                         else
                           sorted = [configs[i]].concat(sorted);
                    }

                    for (var j = 0; j < sorted.length; ++j ) {
                        editor.imageList.moveAsset(
                                    editor.imageList.indexOf(sorted[j]),
                                    editor.imageList.images.length-1);
                    }
                    editor.imageList.endRecordChanges();
                }
            }
        ]
    }

    DragZone {
        id : dragZone

        anchors.fill : parent
        dragDelegate : Item {
            id : dragItem

            property var dragDocs : dragItem.DragZone.dragZone.dragDocuments
            Repeater {
                id : repeater
                model : dragDocs.length > 1 ? 2 : dragDocs
                property Document topDoc : dragDocs[count-1] ? dragDocs[count-1]
                                                             : null
                Item {
                    id : container

                    implicitWidth : 120
                    implicitHeight : 120

                    x : index * -5 // shift to left
                    y : index * 5 // shift to bottom

                    Rectangle { // background
                        opacity :  0.75
                        anchors.fill : parent
                    }

                    Rectangle { // border
                        border {
                            width : 1
                            color : UI.color(UI.SecondaryBase)
                        }
                        color : "transparent"
                        opacity : dragDocs[index] === repeater.topDoc ? 1.0
                                                                      : 0.85
                        anchors.fill : parent
                    }

                    Image {
                        id : thumb
                        anchors {
                            fill : parent
                            margins : 6
                        }
                        sourceSize.width : width*2
                        sourceSize.height : height*2
                        source : dragDocs[dragDocs.length-1].asset ?
                                     dragDocs[dragDocs.length-1].asset.thumbnail
                                   : ""
                        fillMode : Image.PreserveAspectFit
                        visible : dragDocs[index] === repeater.topDoc ? true
                                                                      : false
                        opacity :  0.75
                    }
                }
            }

            Item {
                id : dragCount

                visible : dragDocs.length > 1 ? true : false
                y : 7
                x : repeater.itemAt(repeater.count-1)
                    ? repeater.itemAt(repeater.count-1).implicitWidth - 7
                    : 0

                Rectangle {
                    id : badgeBg
                    width : badge.implicitWidth > 21 ? badge.implicitWidth : 21
                    height : 21
                    opacity : 0.7
                    anchors.right : parent.right
                    anchors.top : parent.top
                }

                Text {
                    id : badge

                    color : UI.color(UI.SecondaryBase)
                    anchors.centerIn : badgeBg
                    font {
                        pixelSize : 16
                        family : UI.fontFamily(UI.PrimaryFont)
                        weight : UI.fontWeight(UI.PrimaryFont)
                    }
                    text : dragDocs.length
                    opacity : 1
                }
            }
        } // dragDelegate


        SelectionGridView {
            id : gridView

            readonly property int margin : 1
            readonly property int propertyNameHeight : 23
            readonly property int rows : Math.ceil(count / cellsPerRow)

            interactive : false
            boundsBehavior : Flickable.StopAtBounds
            width : parent.width - 2 * margin
            height : cellHeight * rows + 2 * margin
            x : margin
            y : margin

            cellWidth : 120
            cellHeight : editor.displayPropertyName === "" ? 120
                                                    : 120 + propertyNameHeight

            onDeletePressed : removeSelection();
            onEscapePressed : gridView.focus = false;

            activeFocusOnTab : true

            Keys.onBacktabPressed : {
                var prev = nextItemInFocusChain(false);
                if (prev)
                    prev.focus = true;
                gridView.selection = [];

            }
            Keys.onTabPressed : {
                var next = nextItemInFocusChain(true);
                if (next)
                    next.focus = true;
                gridView.selection = [];
            }

            model : DocumentListPropertyModel {
                document : editor.imageList
                property : "images"
            }

            DropIndicator {
                id : dropIndicator
                height : gridView.cellHeight
                horizontal : false
            }

            DropIndicator {
                id : dropIndicator2
                height : gridView.cellHeight
                horizontal : false
            }

            SelectionGridViewDropHandler {
                id : dropZone

                z : -1
                type : "configuration:image"
                anchors.fill : parent
                dropIndicator : dropIndicator
                dropIndicator2 : dropIndicator2
                context : imageList.context
                gridView : gridView
            }

            delegate : SelectionGridDelegate {
                id : delegate

                // accessors for context menu actions
                readonly property int                modelIndex  : index
                readonly property ImageConfiguration imageConfig : document

                readonly property bool activeDisplayProperty :
                                            editor.displayPropertyName !== ""
                width : 118
                height : activeDisplayProperty ?
                             118 + gridView.propertyNameHeight : 118

                acceptedButtons : Qt.RightButton | Qt.LeftButton
                hoverEnabled : false
                enabled : editor.selectableItems
                Connections {
                    target : mouseArea
                    onPressed : {
                        dragZone.dragDocuments = gridView.selection;
                    }
                    onClicked : {
                        if (gridView.selection.length === 1)
                            editor.selectedImage = document;

                        // right mouse button pressed
                        if (mouse.button === Qt.RightButton) {
                            menu.targetItem = delegate;
                            menu.clickPosition = delegate.mapToItem(
                                        editor, mouse.x, mouse.y);
                            menu.show();
                        }
                    }
                    onDoubleClicked : {
                        if (mouse.button === Qt.LeftButton)
                            editor.doubleClicked(idx);
                    }

                    ignoreUnknownSignals : true
                }

                Item {
                    id : item
                    width : 118
                    height : parent.height

                    readonly property ImageAsset asset :
                        delegate.imageConfig ? delegate.imageConfig.asset : null
                    readonly property bool isSelected :
                        (gridView.selection.indexOf(document) !== -1) ? true
                                                                      : false

                    Item {
                        width : 118
                        height : 118

                        Rectangle {
                            anchors.centerIn : parent
                            width : 118
                            height : width

                            border.width : 1
                            border.color : modelIndex === currentPageIndex ?
                                               UI.color(UI.PrimaryPress)
                                             : UI.color(UI.PrimaryBase)

                            color : UI.color(UI.PrimaryBase)

                            Behavior on border.color {
                                ColorAnimation {
                                    duration : 500
                                }
                            }
                        }
                        Rectangle {
                            anchors.centerIn : parent
                            width : item.isSelected ? 118 : 110
                            height : width
                            color : item.isSelected ? UI.color(UI.PrimaryPress)
                                    : UI.color(UI.PrimarySeparationLine)

                            Rectangle {
                                color : UI.color(UI.PrimaryBase)
                                anchors.centerIn : parent
                                width : 108
                                height : width
                            }
                            Behavior on width {
                                NumberAnimation {
                                    duration : 100
                                }
                            }
                            Behavior on color {
                                ColorAnimation {
                                    duration : 100
                                }
                            }
                        }
                        Image {
                            anchors.fill : parent
                            anchors.margins : 5
                            fillMode : Image.PreserveAspectFit
                            source : item.asset ? item.asset.thumbnail : ""
                        }
                    }

                    Text {
                        anchors.bottom : parent.bottom
                        anchors.horizontalCenter : parent.horizontalCenter
                        width  : parent.width - 8
                        height : gridView.propertyNameHeight
                        text   : imageConfig && delegate.activeDisplayProperty ?
                                 imageConfig.document[editor.displayPropertyName]
                                   : ""
                        elide  : Text.ElideRight
                        font {
                            family : UI.fontFamily(UI.PrimaryFont)
                            weight : UI.fontWeight(UI.PrimaryFont)
                            pixelSize : 13
                        }
                        visible : delegate.activeDisplayProperty
                    }
                }
            } // delegate
        } // gridView
    } // dragZone
}
