//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick         2.4
import QtMultimedia    5.4
import QtQuick.Layouts 1.1


import wesual.Showcase          1.0
import wesual.Controls          1.0
import wesual.Ui                1.0
import wesual.data.Core         1.0

SidebarItem {
    id : editor

    property VideoConfiguration         videoConfig

    // doubleClicked singal for InlineEditor activation
    signal  doubleClicked()

    // social sharing is disabled for the near feature
//    readonly property string socialShareQml : "
//        import QtQuick 2.3

//        import wesual.Editor 1.0

//        SocialShareButton {
//            id : shareButton
//            document : videoConfig.asset
//            anchors.left : videoThumb.right
//            anchors.leftMargin : 10
//        }"

    panel : SidebarItem.RightScrollablePanel
    active : editor.videoConfig !== null
    sortIndex : 0
    width : parent ? parent.width : 100
    height : items.height

    ContextMenu {
        id : menu

        items : [
            Action {
                text : qsTrId("menu.SideBarImage.Remove")
                iconName : "remove"
                onTriggered : {
                    if(menu.targetItem) {
                        editor.videoConfig.beginRecordChanges();
                        editor.videoConfig.document = null;
                        editor.videoConfig.endRecordChanges();
                    }
                }
            }
        ]
    }


    // prevent deselection of the current  on click outside of controls
    MouseArea {
        id : stealClickFromEditor
        width : items.width
        height : items.height
        onClicked : {
            editor.focus = true;
        }
    }

    Column {
        id : items
        width : parent.width
        spacing : 9
        height : childrenRect.height

        Item {
            id : videoItem

            width :  parent.width
            height : videoThumb.height

            Image {
                id : videoThumb
                width : 110
                height : 110
                fillMode : Image.PreserveAspectFit
                source : editor.videoConfig && editor.videoConfig.asset
                         ? editor.videoConfig.asset.thumbnail : ""
            }
            Rectangle {
                id : thumbBorder
                anchors.fill : videoThumb
                color : "transparent"
                opacity : 0.2
                border.width : 1
                border.color : UI.color(UI.ThumbOverlayAndBorder)


                MouseArea {
                    id : thumbMouse
                    anchors.fill : parent
                    acceptedButtons : Qt.LeftButton | Qt.RightButton
                    onClicked : {
                        editor.focus = true;

                        if (mouse.button === Qt.RightButton) {
                            menu.targetItem = videoThumb;
                            menu.clickPosition = videoThumb.mapToItem(
                                        editor, mouse.x, mouse.y);
                            menu.show();
                        }
                    }
                    onDoubleClicked : {
                        if (mouse.button === Qt.LeftButton)
                            editor.doubleClicked();
                    }
                }

            }

            // social sharing is disabled for the near feature
//            Component.onCompleted :
//                Qt.createQmlObject(socialShareQml, videoItem)
        }

        GridLayout {
            id : grid
            columns : 2
            columnSpacing : 19
            rowSpacing : 14

            UiLabel {
                text : qsTrId("label.Name")
                lineHeightMode : Text.FixedHeight
                lineHeight : 16
                verticalAlignment : Text.AlignBottom
                Layout.alignment : Qt.AlignTop
            }
            UiText {
                text : videoConfig.asset ? videoConfig.asset.fileName : ""
            }

            UiLabel {
                text : qsTrId("label.Duration")
                lineHeightMode : Text.FixedHeight
                lineHeight : 16
                verticalAlignment : Text.AlignBottom
                Layout.alignment : Qt.AlignTop
            }
            UiText {
                text : {
                    if (!videoConfig.asset)
                        return "";

                    var mins  = 0;
                    var hours = 0;
                    var secs  = Math.floor(videoConfig.asset.duration / 1000);

                    if (secs > 60) {
                        mins = Math.floor(secs / 60);

                        if (mins > 60) {
                            hours = Math.floor(mins / 60);
                            mins -= hours * 60;
                        }

                        secs -= mins * 60;
                    }

                    var duration = "";

                    if (hours > 0)
                        duration += (hours < 10) ? "0" + hours + ":" :
                                                   hours + ":";
                    if (mins > 0)
                        duration += (mins < 10) ? "0" + mins + ":" :
                                                  mins + ":";

                    duration += (secs < 10 && mins > 0) ? "0" + secs : secs;

                    return duration;
                }
            }

            UiLabel {
                text : qsTrId("label.VideoPlayMode")
                lineHeightMode : Text.FixedHeight
                lineHeight : 16
                verticalAlignment : Text.AlignBottom
                Layout.alignment : Qt.AlignTop
            }

            UiRadioButtonColumn {
                id : playSettings

                currentIndex : editor.videoConfig &&
                               videoConfig.loops === MediaPlayer.Infinite ?
                               0 : 1
                UiRadioButton {
                    text : qsTrId("label.VideoInfinitePlayLoop")
                    onClicked : {
                        editor.videoConfig.beginRecordChanges();
                        editor.videoConfig.loops = MediaPlayer.Infinite;
                        editor.videoConfig.endRecordChanges();
                    }
                }
                UiRadioButton {
                    text : qsTrId("label.VideoPlayOnce")
                    onClicked : {
                        editor.videoConfig.beginRecordChanges();
                        editor.videoConfig.loops = 1;
                        editor.videoConfig.endRecordChanges();
                    }
                }

                onCurrentIndexChanged : {
                    if (!editor.videoConfig)
                        return;

                    currentIndex = Qt.binding(function() {
                        return editor.videoConfig.loops === MediaPlayer.Infinite
                                                                        ? 0 : 1;
                    });
                }
            }
        }

        UiCheckBox {
            text : qsTrId("label.PlayWholeVideoInAutopilot")

            checked : editor.videoConfig.willDelay

            onClicked : {
                editor.videoConfig.beginRecordChanges();
                editor.videoConfig.willDelay = checked;
                editor.videoConfig.endRecordChanges();
            }
        }

        Item {
            width  : parent.width
            height : muteButton.height

            UiToolButton {
                id : muteButton

                iconSize  : UiIcon.Icon_32x32
                iconName  : {
                    if (!videoConfig.asset ||
                        !videoConfig.asset.hasAudio ||
                        videoConfig.volume === 0 ||
                        videoConfig.muted)
                        return "mute";
                    else {
                        var level2 = scaleSlider.maximumValue / 3;
                        var level3 = (2 * scaleSlider.maximumValue) / 3;

                        if (videoConfig.volume < level2)
                            return "audio-level-1";
                        else if (videoConfig.volume >= level2 &&
                                 videoConfig.volume < level3)
                            return "audio-level-2";
                        else
                            return "audio-level-3";
                    }
                }
                enabled   : scaleSlider.enabled
                onClicked : {
                    videoConfig.beginRecordChanges();
                    videoConfig.muted = !videoConfig.muted;
                    videoConfig.endRecordChanges();
                }
            }

            UiSlider {
                id : scaleSlider

                anchors {
                    left : muteButton.right
                    leftMargin : 6
                    verticalCenter : parent.verticalCenter
                }
                valueBarVisible : true
                valueBarOrigin  : minimumValue
                width        : 200
                height       : 20
                minimumValue : 0.0
                maximumValue : 1.0
                enabled      : videoConfig.asset && videoConfig.asset.hasAudio
                visible      : enabled

                onPressedChanged : {
                    if (pressed) {
                        videoConfig.beginRecordChanges();
                        videoConfig.muted = false;
                    } else {
                        videoConfig.endRecordChanges();
                    }
                }

                Binding {
                    target   : videoConfig
                    when     : scaleSlider.pressed
                    property : "volume"
                    value    : scaleSlider.value
                }
                Binding {
                    target   : scaleSlider
                    when     : !scaleSlider.pressed
                    property : "value"
                    value    : videoConfig.volume
                }
            }

            UiText {
                text    : qsTrId("label.NoAudio")
                visible : !scaleSlider.enabled
                color   : UiColors.getColor(UiColors.MediumLightGrey27)
                anchors {
                    left : muteButton.right
                    verticalCenter : parent.verticalCenter
                }
            }
        }
    }
}
