//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.2

import wesual.Showcase 1.0
import wesual.Controls 1.0

import wesual.data.Core 1.0

Item {
    id : thumbnailStrip

    property int tileWidth  : 200
    property int tileHeight : 200

    property ImageListConfiguration imageList

    // Aliases
    property alias orientation : listView.orientation
    property alias tileSpacing : listView.spacing

    readonly property real scrollRatio : {
        var index = imageList.currentIndex;
        if (index === -1)
            index = 0;

        return index / (imageList.images.length - 1);
    }
    readonly property bool overflow : listView.overflow

    width  : orientation === Qt.Vertical ? tileWidth : parent.width
    height : orientation === Qt.Horizontal ? tileHeight : parent.height

    Connections {
        target : imageList
        ignoreUnknownSignals : true
        onCurrentImageChanged : listView.centerCurrent()
    }

    Connections {
        target : imageList
        ignoreUnknownSignals : true
        onCurrentIndexChanged : listView.centerCurrent()
    }

    DragListView {
        id : listView

        readonly property bool overflow : {
            var avail = 0;
            if (orientation === Qt.Horizontal) {
                avail = thumbnailStrip.width;
            } else {
                avail = thumbnailStrip.height;
            };

            return extent > avail;
        }

        readonly property int extent : {
            var tile = orientation === Qt.Horizontal ? tileWidth : tileHeight;
            return count * tile + (count - 1) * spacing;
        }

        function centerCurrent() {
            if (!overflow || dragDocument !== null)
                return;

            var index = imageList.currentIndex;
            if (index === -1)
                return;

            if (orientation === Qt.Horizontal) {
                var offset = index * (tileWidth + spacing);
                var center = listView.width / 2.0 - tileWidth / 2.0;

                listView.contentX = listView.originX + offset - center;
            } else {
                var offset = index * (tileHeight + spacing);
                var center = listView.height / 2.0 - tileHeight / 2.0;

                listView.contentY = listView.originY + offset - center;
            }
        }

        model : DocumentListPropertyModel {
            document : imageList
            property : "images"
        }

        spacing : 20
        pixelAligned : true
        cacheBuffer : count
        displayMarginBeginning : Math.max(tileWidth, tileHeight) * 2
        displayMarginEnd : displayMarginBeginning

        onDocumentDropped : {
            imageList.currentImage = dropped;
            centerCurrent();
        }

        states : [
            State {
                name : "condensed-horizontal"
                when : !listView.overflow &&
                       listView.orientation === Qt.Horizontal
                PropertyChanges {
                    target : listView
                    width : extent
                    height : thumbnailStrip.tileHeight
                }
                AnchorChanges {
                    target : listView
                    anchors.horizontalCenter : parent.horizontalCenter
                }
            },
            State {
                name : "condensed-vertical"
                when : !listView.overflow &&
                       listView.orientation === Qt.Vertical
                PropertyChanges {
                    target : listView
                    height : extent
                    width : thumbnailStrip.tileWidth
                }
                AnchorChanges {
                    target : listView
                    anchors.verticalCenter : parent.verticalCenter
                }
            },
            State {
                name : "full-horizontal"
                when : listView.overflow &&
                       listView.orientation === Qt.Horizontal
                PropertyChanges {
                    target : listView
                    width : parent.width
                    height : thumbnailStrip.tileHeight
                    leftMargin : width / 2.0 - thumbnailStrip.tileWidth / 2.0
                    rightMargin : leftMargin
                }
                AnchorChanges {
                    target : listView
                    anchors.left : parent.left
                }
            },
            State {
                name : "full-vertical"
                when : listView.overflow && listView.orientation === Qt.Vertical
                PropertyChanges {
                    target : listView
                    height : parent.height
                    width : tileWidth
                    topMargin : height / 2.0 - tileHeight / 2.0
                    bottomMargin : topMargin
                }
                AnchorChanges {
                    target : listView
                    anchors.top : parent.top
                }
            }
        ]

        delegate : DragDelegate {
            id : dragDelegate

            readonly property bool activeItem :
                thumbnailStrip.imageList.currentImage === document

            width : tileWidth
            height : tileHeight

            Rectangle {
                id : wrapper

                readonly property bool draggedItem :
                    !parent.isDragged && listView.dragDocument === document

                width : tileWidth
                height : tileHeight
                color : dragDelegate.activeItem ? "black" : "white"
                opacity : draggedItem ? .2 : 1

                Behavior on color {
                    ColorAnimation {
                        duration : 200
                    }
                }
                Behavior on opacity {
                    NumberAnimation {
                        duration : 200
                    }
                }

                ImageBox {
                    anchors.fill : parent
                    anchors.margins : 10
                    configuration : document
                }

                MouseArea {
                    anchors.fill : parent
                    enabled : !dragDelegate.activeItem
                    onClicked : {
                        thumbnailStrip.imageList.currentImage = document;
                    }
                }
            }
        }
    }
}
