//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.12

import wesual.Ui 1.0

import "private"

/*!
\qmltype     UiCondensedTextEdit
\inqmlmodule we.captivo.Ui
\since       we.captivo.Ui 1.0
\inherits    UiTextEditBase
\ingroup     captivo-ui-controls

\brief A visually condensed editor for multi-line text.

The UiCondensedTextEdit provides an edit control to edit a multiple lines of
plain text. To allow for more compact form layouts, the frame around the
control is only displayed when the control is hovered or has the keyboard
focus.

In addition, the control border does not contribute to the control geometry.
Positions are specified relative to the contained text.

\sa UiTextEdit
*/

UiTextEditBase {
    id : uiLineEdit

    verticalMargin : 4

    Rectangle {
        id : background

        readonly property int horizontalMargin : 6

        color        : UiColors.getColor(UiColors.White)
        border.width : 1
        z            : -1
        opacity      : 0

        x      : -horizontalMargin
        width  : parent.width + 2 * horizontalMargin
        height : parent.height

        states : [
            State {
                name : "focus"
                when : uiLineEdit.activeFocus

                PropertyChanges {
                    target       : background
                    border.color : UiColors.getColor(UiColors.SelectionGreen)
                    opacity      : 1
                }
            },
            State {
                name : "hover"
                when : uiLineEdit.hover

                PropertyChanges {
                    target       : background
                    border.color : UiColors.getColor(UiColors.HoverGreen)
                    opacity      : 1
                }
            }
        ]
    }
}
